/**
 * Create a panel containing a notification message and a link. The panel slides into the document from the top and
 * disappears automatically after a short duration.
 *
 * The .uiSlidePanel() creates a panel to slide in a notification message.
 *
 * ### Examples
 * Create a slide panel with all possible configuration options.
 *
 * HTML:
 * <div id="slidePanelId"></div>
 *
 * JavaScript:
 *
 *    <script>
 *        require([
 *           'jquery',
 *           'ep/ui/slide-panel',
 *
 *           '$ready!'
 *       ], function($, ep) {
 *            ep('#slidePanelId').uiSlidePanel({
 *                duration: 5000,
 *                rootCssClass: 'add-to-cart-notice',
 *                type: 'success',
 *                visible: false,
 *                notification: 'Item added to cart.',
 *                link: {
 *                    text: 'View cart',
 *                    href: '/cart'
 *                }
 *            });
 *       });
 *   </script>
 *
 *
 * ### Examples
 * Set options:
 * - type:         (string) success, failure
 * - notification: (string) notification message in panel
 * - link:         (object) {text: link-text, href: url}
 *
 * HTML:
 * <div id="slidePanelId"></div>
 *
 * JavaScript:
 *
 *    <script>
 *        require([
 *           'jquery',
 *           'ep/ui/slide-panel',
 *
 *           '$ready!'
 *       ], function($, ep) {
 *            var slidePanel = ep('#slidePanelId').uiSlidePanel();
 *
 *            // set "success" style
 *            slidePanel.uiSlidePanel('option', 'type', 'succes');
 *            // set notification message
 *            slidePanel.uiSlidePanel('option', 'notification', 'Item added to cart.');
 *            // set link options
 *            slidePanel.uiSlidePanel('option', 'link', {text: 'View cart', href: '/cart'});
 *       });
 *   </script>
 *
 *
 * ### Examples
 * Show/hide slide panel.
 *
 * HTML:
 * <div id="slidePanelId"></div>
 *
 * JavaScript:
 *
 *    <script>
 *        require([
 *           'jquery',
 *           'ep/ui/slide-panel',
 *
 *           '$ready!'
 *       ], function($, ep) {
 *            var slidePanel = ep('#slidePanelId').uiSlidePanel();
 *
 *            slidePanel.uiSlidePanel('show');
 *            slidePanel.uiSlidePanel('hide');
 *       });
 *   </script>
 *
 *
 *
 * @class jQuery.ui.uiSlidePanel
 * @extends jQuery.widget
 *
 * @uses jquery
 * @uses ep
 * @uses $tmpl!./slide-panel
 * @uses $dict!ep/dict
 * @uses jquery/ui/widget
 * @uses jquery/tmpl
 * @since 7.43.0
 */

/**
 * @cfg {Integer} [duration] Time to wait until the panel disappears.
 */

/**
 * @cfg {String} [rootCssClass] Root css class of the template wrapper.
 */

/**
 * @cfg {String} [type] Style of the panel (e.g. success, failure).
 */

/**
 * @cfg {Boolean} [visible] Flag to decide if the panel should be visible (true) or hidden (false) on initialization.
 */

/**
 * @cfg {String} [notification] Notification message in panel.
 */

/**
 * @cfg {Object} [link] Object to describe the link. (structure: {text: [String], href: [String]}
 */

/**
 * See `jQuery.ui.uiSlidePanel` for details.
 *
 * @param {Integer} [duration] Time to wait until the panel disappears.
 * @param {String} [rootCssClass] Root css class of the template wrapper.
 * @param {String} [type] Style of the panel (e.g. success, failure).
 * @param {Boolean} [visible] Flag to decide if the panel should be visible (true) or hidden (false) on initialization.
 * @param {String} [notification] Notification message in panel.
 * @param {Object} [link] Object to describe the link. (structure: {text: [String], href: [String]}
 *
 *
 * @method uiSlidePanel
 * @member jQuery
 *
 * @since 7.43.0
 */

/*
 * @copyright       © Copyright 2006-2020, epages GmbH, All Rights Reserved.
 *
 * @module          ep/ui/slide-panel
 */

define("ep/ui/slide-panel", [
    "jquery",
    "ep",
    "$tmpl!./slide-panel",
    "$dict!ep/dict",

    "jquery/ui/widget",
    "jquery/tmpl"
], function($, ep, tmplSlidePanel, epDict) {
/*
 * @dictionary      ep.dict
 *
 * @translation     {Close}
 *                  {GoToCartLink}
 *                  {ProductAddedToCart}
 *
 */

    // translation object
    var translations = {
            Close: epDict.translate('Close'),
            GoToCartLink: epDict.translate('GoToCartLink'),
            ProductAddedToCart: epDict.translate('ProductAddedToCart')
        },

        types = ['success', 'failure'];

    $.widget('ui.uiSlidePanel', {
    //  panel,
    //  hTimer,

        tmplElements: {},

        options: {
            duration: 5000,
            rootCssClass: 'add-to-cart-notice',
            type: 'success', /* failure */
            visible: false,
            notification: translations.ProductAddedToCart,
            link: {
                text: translations.GoToCartLink,
                href: "/cart"
            }
        },

        /**
         * Method is called during initialization, when widget is created.
         *
         * @member jQuery.ui.uiSlidePanel
         *
         * @since 7.43.0
         */
        _create: function() {
            var self = this,
                o = self.options,
                elem = self.element;

            // add rootCssClass class
            elem.addClass(o.rootCssClass);

            // set visbility/type classes
            self._setType();
            self._setVisibility(o.visible);

            // create UI
            self._createPanel();

            // append element to the end of the document
            $('body').append(elem);
        },

        /**
         * Method sets the type-css-class in the class attribute.
         *
         * @member jQuery.ui.uiSlidePanel
         *
         * @since 7.43.0
         */
        _setType: function(type) {
            var self = this,
                o = self.options,
                elem = self.element;

            type = type ? type : o.type;

            // remove all type css classes
            types.forEach(function(itemType) {
                elem.removeClass(o.rootCssClass + '-' + itemType);
            });

            // add new type
            elem.addClass(o.rootCssClass + '-' + type);
        },

        /**
         * Method sets the visibility-css-class in the class attribute.
         *
         * @member jQuery.ui.uiSlidePanel
         *
         * @since 7.43.0
         */
        _setVisibility: function(isVisible) {
            var self = this,
                o = self.options,
                elem = self.element;

            elem.removeClass(isVisible ? o.rootCssClass + '-hidden' : o.rootCssClass + '-visible');
            elem.addClass(isVisible ? o.rootCssClass + '-visible' : o.rootCssClass + '-hidden');

            if (isVisible) {
                self.hTimer = window.setTimeout(function() {
                    self._setVisibility(false);
                }, o.duration);
            }
        },

        /**
         * Method changes the content for the notification message and the link inside the widget template.
         *
         * @member jQuery.ui.uiSlidePanel
         *
         * @since 7.43.0
         */
        _setContent: function() {
            var self = this,
                o = self.options,
                elem = self.element;

            // set notification content
            self.tmplElements.notification.html(o.notification);

            if (o.link.text && o.link.href) {
                // set link content
                self.tmplElements.link.text(o.link.text);
                self.tmplElements.link.attr('href', o.link.href);
                self.tmplElements.link.show();
            } else {
                self.tmplElements.link.hide();
            }
        },

        /**
         * jQuery widget method to handle options.
         *
         * @member jQuery.ui.uiSlidePanel
         *
         * @since 7.43.0
         */
        _setOption: function(key, value) {
            var self = this,
                o = self.options,
                elem = self.element;

            self._super(key, value);

            switch(key.toLowerCase()) {
                case 'type':
                    // add type to types array if not inclued yet
                    if (types.indexOf(value) === -1) {
                        types.push(value);
                    }

                    // set type (success, failure ...)
                    self._setType(value);

                    break;

                case 'notification':
                case 'link':
                    // set content
                    self._setContent();

                    break;
            }

        },

        /**
         * Method renders the template and creates the panel.
         *
         * @member jQuery.ui.uiSlidePanel
         *
         * @since 7.43.0
         */
        _createPanel: function() {
            var self = this,
                o = self.options,
                elem = self.element,
                renderedTmpl;

            renderedTmpl = tmplSlidePanel([{
                rootCssClass: o.rootCssClass,
                dict: translations
            }]);

            // get DOM-Elements from template
            $.extend(self.tmplElements, renderedTmpl.tmplItem('elements'));
            // initialize dialog widget
            self.panel = elem.html(renderedTmpl);

            // set initial content
            self._setContent();

            // handle click on hide button
            self.tmplElements.hideButton.on('click.hideNotification', function() {
                self.hide();
            });
        },

        /**
         * When the method is called the panel slides into the document.
         *
         * @method open
         * @member jQuery.ui.uiSlidePanel
         *
         * @since 7.43.0
         */
        show: function() {
            var self = this;

            if (!self.panel) {
                self._createPanel();
            }

            self._setVisibility(true);
        },

        /**
         * When the method is called the panel will be hidden
         *
         * @method close
         * @member jQuery.ui.uiSlidePanel
         *
         * @since 7.43.0
         */
        hide: function() {
            var self = this;

            if (self.panel) {
                window.clearTimeout(self.hTimer);
                self._setVisibility(false);
            }
        },

        destroy: function() {
            var self = this;

            // unregister events
            self.tmplElements.hideButton.off('click.hideNotification');
            // remove panel from DOM
            self.element.remove();

            this._superApply(arguments);
        }
    });

    return ep;
});


